### This R code allows to reproduce the main results presented in the paper - R.3.4.4 ###

### Libraries

# Packages installation - Bioconductor 3.5 (BiocInstaller 1.26.1)
source("https://bioconductor.org/biocLite.R")
biocLite("INSPEcT")
biocLite("topGO")
biocLite("rGREAT")
biocLite("GenomicFeatures")
biocLite("biomaRt")
biocLite("TxDb.Mmusculus.UCSC.mm9.knownGene")
biocLite("org.Mm.eg.db")
biocLite("compEpiTools")
install.packages("pheatmap")

# Packages and objects loading
library(INSPEcT)
library(topGO)
library(rGREAT)
library(GenomicFeatures)
library(biomaRt)
library(pheatmap)
library(compEpiTools)
require(TxDb.Mmusculus.UCSC.mm9.knownGene)
require("org.Mm.eg.db")

txdb <- TxDb.Mmusculus.UCSC.mm9.knownGene
mart = useMart('ENSEMBL_MART_ENSEMBL',dataset='mmusculus_gene_ensembl', host="may2012.archive.ensembl.org")

### Custom functions

source("File S2.R")

### Evaluation of the kinetic rates from total and nascent RNA-seq data
## BAMs

total_bamWT <-  # URLs for the BAMs of the total RNA in Wild Type cells: 0,15,30,45,60,75 minutes
names(total_bamWT) <- c('Input WT 0 min IL-7','Input WT 15min IL-7','Input WT 45min IL-7','Input WT 60min IL-7','Input WT 75min IL-7')
total_bamKO <-  # URLs for the BAMs of the total RNA in KnockOut cells: 0,15,30,60 minutes
names(total_bamKO) <- c('Input KO 0min IL-7','Input KO 15min IL-7','Input KO 60min IL-7')
fsu_bamWT <- # URLs for the BAMs of the nascent RNA in Wild Type cells: 0,15,45,60,75 minutes
names(fsu_bamWT) <- c('Enriched WT 0min IL-7','Enriched WT 15min IL-7','Enriched WT 45min IL-7','Enriched WT 60min IL-7' ,'Enriched WT 75min IL-7'  )
fsu_bamKO <-  # URLs for the BAMs of the nascent RNA in KnockOut cells: 0,15,60 minutes
names(fsu_bamKO) <-  c('Enriched KO 0min IL-7','Enriched KO 15min IL-7','Enriched KO 60min IL-7')

## RPKMs quantification

makeRPKMsOutWT <- makeRPKMs(txdb, fsu_bamWT, total_bamWT)
makeRPKMsOutKO <- makeRPKMs(txdb, fsu_bamKO, total_bamKO)

## Results of the RPKMs quantification loading and formatting

makeRPKMsOutKO <- readRDS("File S3.rds")
makeRPKMsOutWT <- readRDS("File S4.rds")

totalIntronsWT <- makeRPKMsOutWT$rpkms$total_introns
totalExonsWT <- makeRPKMsOutWT$rpkms$total_exons

foursuIntronsWT <- makeRPKMsOutWT$rpkms$foursu_introns
foursuExonsWT <- makeRPKMsOutWT$rpkms$foursu_exons

totalIntronsKO <- makeRPKMsOutKO$rpkms$total_introns
totalExonsKO <- makeRPKMsOutKO$rpkms$total_exons

foursuIntronsKO <- makeRPKMsOutKO$rpkms$foursu_introns
foursuExonsKO <- makeRPKMsOutKO$rpkms$foursu_exons

## Expressions correlation analysis, Figure S1

pheatmap(cor(cbind(totalWT=totalIntronsWT
				  ,totalKO=totalIntronsKO
				  ,nascentWT=foursuIntronsWT
				  ,nascentKO=foursuIntronsKO),method="s"))

pheatmap(cor(cbind(totalWT=totalExonsWT
				  ,totalKO=totalExonsKO
				  ,nascentWT=foursuExonsWT
				  ,nascentKO=foursuExonsKO),method="s"))

## Removal of the time 30 minutes time points

totalIntronsWT <- totalIntronsWT[,-3]
totalExonsWT <- totalExonsWT[,-3]
foursuIntronsWT <- foursuIntronsWT[,-3]
foursuExonsWT <- foursuExonsWT[,-3]
totalIntronsKO <- totalIntronsKO[,-3]
totalExonsKO <- totalExonsKO[,-3]
foursuIntronsKO <- foursuIntronsKO[,-3]
foursuExonsKO <- foursuExonsKO[,-3]

## Rates first guess inference

# Time points
tptsWT <- c(0,0.25,3/4,1,1.25)
tptsKO <- c(0,0.25,1)

# Labeling time
tL <- 1/4

inspectObjectWT <- newINSPEcT(tptsWT
							, tL
							, foursuExonsWT
							, totalExonsWT
							, foursuIntronsWT
							, totalIntronsWT
							, BPPARAM=SerialParam())

inspectObjectKO <- newINSPEcT(tptsKO
							, tL
							, foursuExonsKO
							, totalExonsKO
							, foursuIntronsKO
							, totalIntronsKO
							, BPPARAM=SerialParam())

## Expressions filters on the experimental data

# Total RNA
totalKO <- ratesFirstGuess(inspectObjectKO,"total")
totalWT <- ratesFirstGuess(inspectObjectWT,"total")
# Premature RNA
prematureKO <- ratesFirstGuess(inspectObjectKO,"preMRNA")
prematureWT <- ratesFirstGuess(inspectObjectWT,"preMRNA")
# Nascent RNA
synthesisKO <- ratesFirstGuess(inspectObjectKO,"synthesis")
synthesisWT <- ratesFirstGuess(inspectObjectWT,"synthesis")

boolKO <- apply(synthesisKO,1,function(r)all(r>=1))&apply(totalKO,1,function(r)all(r>=1))&apply(prematureKO,1,function(r)all(r>=0.1))
boolWT <- apply(synthesisWT,1,function(r)all(r>=1))&apply(totalWT,1,function(r)all(r>=1))&apply(prematureWT,1,function(r)all(r>=0.1))
genes <- intersect(names(boolKO),names(boolWT))
bool <- boolWT[genes] & boolKO[genes] 

# Reduction of the INSPEcT objects
inspectObjectKO <- inspectObjectKO[genes][bool]
inspectObjectWT <- inspectObjectWT[genes][bool]

## Data preparation for the analysis

# Total RNA
totalKO <- ratesFirstGuess(inspectObjectKO,"total")
totalWT <- ratesFirstGuess(inspectObjectWT,"total")
# Premature RNA
prematureKO <- ratesFirstGuess(inspectObjectKO,"preMRNA")
prematureWT <- ratesFirstGuess(inspectObjectWT,"preMRNA")
# Synthesis rate
synthesisKO <- ratesFirstGuess(inspectObjectKO,"synthesis")
synthesisWT <- ratesFirstGuess(inspectObjectWT,"synthesis")
# Prodcessing rate
processingKO <- ratesFirstGuess(inspectObjectKO,"processing")
processingWT <- ratesFirstGuess(inspectObjectWT,"processing")
# Degradation rate
degradationKO <- ratesFirstGuess(inspectObjectKO,"degradation")
degradationWT <- ratesFirstGuess(inspectObjectWT,"degradation")

# Percentages of not defined rates
round(table(is.finite(processingWT))/sum(table(is.finite(processingWT))),3)
round(table(is.finite(degradationWT))/sum(table(is.finite(degradationWT))),3)

round(table(is.finite(processingKO))/sum(table(is.finite(processingKO))),3)
round(table(is.finite(degradationKO))/sum(table(is.finite(degradationKO))),3)

# Interpolation
processingWT <- t(apply(processingWT,1,function(r){impute_na_tc(tpts=tpts(inspectObjectWT),tcdata=r)}))
degradationWT <- t(apply(degradationWT,1,function(r){impute_na_tc(tpts=tpts(inspectObjectWT),tcdata=r)}))

processingKO <- t(apply(processingKO,1,function(r){impute_na_tc(tpts=tpts(inspectObjectKO),tcdata=r)}))
degradationKO <- t(apply(degradationKO,1,function(r){impute_na_tc(tpts=tpts(inspectObjectKO),tcdata=r)}))

### Identification of m6A+ and m6A- genes from RIP data

# .narrowPeak file from macs2 analysis
rep1 <- read.table('Rep1.narrowPeak', sep='\t', quote='')
rep2 <- read.table('Rep2.narrowPeak', sep='\t', quote='')

# Conversion to GRanges
rep1 <- GRanges(seqnames=rep1[,1], IRanges(rep1[,2], rep1[,3]))
rep2 <- GRanges(seqnames=rep2[,1], IRanges(rep2[,2], rep2[,3]))

# Intersection
rep1_intersect_rep2 <- intersect(rep1, rep2)

# Intersection annotation on the reference genome
intersect_annotation <- GRannotate(GRmidpoint(rep1_intersect_rep2), txdb=txdb, EG2GS=org.Mm.eg.db)

# Methylated genes with a minimum expression level
methGenes <- unique(mcols(intersect_annotation)[,"nearest_gene_id"])
methGenes <- intersect(methGenes,rownames(totalWT))

# Union	
rep1_union_rep2 <- union(rep1, rep2)

# Union annotation on the reference genome
union_annotation <- GRannotate(GRmidpoint(rep1_union_rep2), txdb=txdb, EG2GS=org.Mm.eg.db)

# Gene not methylated in any replicate
noMethGenes <- setdiff(rownames(totalWT),unique(mcols(union_annotation)[,"nearest_gene_id"]))

### Homeostasis analysis - 0 minutes

# Data
	totalKO_0 <- totalKO[,1]
	prematureKO_0 <- prematureKO[,1]
	synthesisKO_0 <- synthesisKO[,1]
	processingKO_0 <- processingKO[,1]
	degradationKO_0 <- degradationKO[,1]

	totalWT_0 <- totalWT[,1]
	prematureWT_0 <- prematureWT[,1]
	synthesisWT_0 <- synthesisWT[,1]
	processingWT_0 <- processingWT[,1]
	degradationWT_0 <- degradationWT[,1]

	totalFC_0 <- log2(totalKO_0/totalWT_0)
	prematureFC_0 <- log2(prematureKO_0/prematureWT_0)
	synthesisFC_0 <- log2(synthesisKO_0/synthesisWT_0)
	processingFC_0 <- log2(processingKO_0/processingWT_0)
	degradationFC_0 <- log2(degradationKO_0/degradationWT_0)

# Boxplots - Figure 1B
	par(mfrow=c(1,3))
	
	boxplot(list(all=synthesisFC_0,"m6a+"=synthesisFC_0[methGenes],"m6A-"=synthesisFC_0[noMethGenes]),outline=FALSE,main="",ylab="synthesis rate Log2(KO/WT)");abline(h=0,lty=3,notch=TRUE)
	boxplot(list(all=processingFC_0,"m6a+"=processingFC_0[methGenes],"m6A-"=processingFC_0[noMethGenes]),outline=FALSE,main="",ylab="processing rate Log2(KO/WT)");abline(h=0,lty=3,notch=TRUE)
	boxplot(list(all=degradationFC_0,"m6a+"=degradationFC_0[methGenes],"m6A-"=degradationFC_0[noMethGenes]),outline=FALSE,main="",ylab="degradation rate Log2(KO/WT)");abline(h=0,lty=3,notch=TRUE)

# Wilcoxon test

	set.seed(1)
		median(sapply(1:1000,function(i)wilcox.test(x=sample(synthesisFC_0, length(methGenes)),mu=0,alternative="less")$p.value))
		wilcox.test(x=synthesisFC_0[methGenes],mu=0,alternative="less")$p.value
		median(sapply(1:1000,function(i)wilcox.test(x=sample(synthesisFC_0[noMethGenes], length(methGenes)),mu=0,alternative="less")$p.value))
	
	set.seed(1)
		median(sapply(1:1000,function(i)wilcox.test(x=sample(processingFC_0, length(methGenes)),mu=0,alternative="less")$p.value))
		wilcox.test(x=processingFC_0[methGenes],mu=0,alternative="less")$p.value
		median(sapply(1:1000,function(i)wilcox.test(x=sample(processingFC_0[noMethGenes], length(methGenes)),mu=0,alternative="less")$p.value))

	set.seed(1)
		median(sapply(1:1000,function(i)wilcox.test(x=sample(degradationFC_0, length(methGenes)),mu=0,alternative="less")$p.value))
		wilcox.test(x=degradationFC_0[methGenes],mu=0,alternative="less")$p.value
		median(sapply(1:1000,function(i)wilcox.test(x=sample(degradationFC_0[noMethGenes], length(methGenes)),mu=0,alternative="less")$p.value))

# Functional analysis

	# Fold changes
	synthesis_FC_0 <- log10(synthesisWT_0) - log10(synthesisKO_0)
	processing_FC_0 <- log10(processingWT_0) - log10(processingKO_0)
	degradation_FC_0 <- log10(degradationWT_0) - log10(degradationKO_0)

	# synthesis UP - WT > KO

		# 100		
			genes_tmp <- names(sort(synthesis_FC_0,decreasing=TRUE))[1:100]
		
			genesRanges_tmp <- select(mart, keys=genes_tmp, columns=c("chromosome_name","start_position","end_position"),keytype="entrezgene")
			colnames(genesRanges_tmp) <- c("chr","start","end")
		
			genesRanges_tmp$chr <- paste0("chr",genesRanges_tmp$chr)
		
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166349",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165754",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166291",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166314",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165789",]
		
			job_tmp <- submitGreatJob(genesRanges_tmp, species = "mm9",request_interval = 0)
			disregulatedGenes_synthesis_up_100 <- getEnrichmentTables(job_tmp, ontology = availableOntologies(job_tmp))

		# 500
			genes_tmp <- names(sort(synthesis_FC_0,decreasing=TRUE))[1:500]
		
			genesRanges_tmp <- select(mart, keys=genes_tmp, columns=c("chromosome_name","start_position","end_position"),keytype="entrezgene")
			colnames(genesRanges_tmp) <- c("chr","start","end")
		
			genesRanges_tmp$chr <- paste0("chr",genesRanges_tmp$chr)
		
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166349",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165754",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166291",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166314",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165789",]
		
			job_tmp <- submitGreatJob(genesRanges_tmp, species = "mm9",request_interval = 0)
			disregulatedGenes_synthesis_up_500 <- getEnrichmentTables(job_tmp, ontology = availableOntologies(job_tmp))
	
	# synthesis DOWN - WT < KO

		# 100
			genes_tmp <- names(sort(synthesis_FC_0,decreasing=FALSE))[1:100]
		
			genesRanges_tmp <- select(mart, keys=genes_tmp, columns=c("chromosome_name","start_position","end_position"),keytype="entrezgene")
			colnames(genesRanges_tmp) <- c("chr","start","end")
		
			genesRanges_tmp$chr <- paste0("chr",genesRanges_tmp$chr)
		
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166349",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165754",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166291",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166314",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165789",]
		
			job_tmp <- submitGreatJob(genesRanges_tmp, species = "mm9",request_interval = 0)
			disregulatedGenes_synthesis_down_100 <- getEnrichmentTables(job_tmp, ontology = availableOntologies(job_tmp))
		
		# 500
			genes_tmp <- names(sort(synthesis_FC_0,decreasing=FALSE))[1:500]
		
			genesRanges_tmp <- select(mart, keys=genes_tmp, columns=c("chromosome_name","start_position","end_position"),keytype="entrezgene")
			colnames(genesRanges_tmp) <- c("chr","start","end")
		
			genesRanges_tmp$chr <- paste0("chr",genesRanges_tmp$chr)
		
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166349",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165754",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166291",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166314",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165789",]
		
			job_tmp <- submitGreatJob(genesRanges_tmp, species = "mm9",request_interval = 0)
			disregulatedGenes_synthesis_down_500 <- getEnrichmentTables(job_tmp, ontology = availableOntologies(job_tmp))
	
	# Significant terms analysis - synthesis

		synthesis_up_100 <- enrichmentsSumUp(disregulatedGenes_synthesis_up_100, th=0.001)
		synthesis_up_500 <- enrichmentsSumUp(disregulatedGenes_synthesis_up_500, th=0.001)
		
		synthesis_down_100 <- enrichmentsSumUp(disregulatedGenes_synthesis_down_100, th=0.001)
		synthesis_down_500 <- enrichmentsSumUp(disregulatedGenes_synthesis_down_500, th=0.001)
	
		synthesis_both <- commonGOterms(a = collapsedGOterms(a = synthesis_up_100,b = synthesis_up_500)
									  , b = collapsedGOterms(a = synthesis_down_100,b = synthesis_down_500))
		synthesis_both
	
		differentialGOterms(collapsedGOterms(a = synthesis_up_100,b = synthesis_up_500),synthesis_both)
	
		differentialGOterms(collapsedGOterms(a = synthesis_down_100,b = synthesis_down_500),synthesis_both)

	# processing UP - WT > KO

		# 100
			genes_tmp <- names(sort(processing_FC_0,decreasing=TRUE))[1:100]
			
			genesRanges_tmp <- select(mart, keys=genes_tmp, columns=c("chromosome_name","start_position","end_position"),keytype="entrezgene")
			colnames(genesRanges_tmp) <- c("chr","start","end")
			
			genesRanges_tmp$chr <- paste0("chr",genesRanges_tmp$chr)
			
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166349",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165754",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166291",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166314",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165789",]
			
			job_tmp <- submitGreatJob(genesRanges_tmp, species = "mm9",request_interval = 0)
			disregulatedGenes_processing_up_100 <- getEnrichmentTables(job_tmp, ontology = availableOntologies(job_tmp))
		
		# 500
			genes_tmp <- names(sort(processing_FC_0,decreasing=TRUE))[1:500]
		
			genesRanges_tmp <- select(mart, keys=genes_tmp, columns=c("chromosome_name","start_position","end_position"),keytype="entrezgene")
			colnames(genesRanges_tmp) <- c("chr","start","end")
		
			genesRanges_tmp$chr <- paste0("chr",genesRanges_tmp$chr)
		
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166349",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165754",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166291",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166314",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165789",]
		
			job_tmp <- submitGreatJob(genesRanges_tmp, species = "mm9",request_interval = 0)
			disregulatedGenes_processing_up_500 <- getEnrichmentTables(job_tmp, ontology = availableOntologies(job_tmp))
	
	# processing DOWN - WT < KO

		# 100
			genes_tmp <- names(sort(processing_FC_0,decreasing=FALSE))[1:100]
		
			genesRanges_tmp <- select(mart, keys=genes_tmp, columns=c("chromosome_name","start_position","end_position"),keytype="entrezgene")
			colnames(genesRanges_tmp) <- c("chr","start","end")
			
			genesRanges_tmp$chr <- paste0("chr",genesRanges_tmp$chr)
			
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166349",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165754",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166291",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166314",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165789",]
			
			job_tmp <- submitGreatJob(genesRanges_tmp, species = "mm9",request_interval = 0)
			disregulatedGenes_processing_down_100 <- getEnrichmentTables(job_tmp, ontology = availableOntologies(job_tmp))
		
		# 500
		
			genes_tmp <- names(sort(processing_FC_0,decreasing=FALSE))[1:500]
		
			genesRanges_tmp <- select(mart, keys=genes_tmp, columns=c("chromosome_name","start_position","end_position"),keytype="entrezgene")
			colnames(genesRanges_tmp) <- c("chr","start","end")
		
			genesRanges_tmp$chr <- paste0("chr",genesRanges_tmp$chr)
		
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166349",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165754",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166291",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166314",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165789",]
		
			job_tmp <- submitGreatJob(genesRanges_tmp, species = "mm9",request_interval = 0)
			disregulatedGenes_processing_down_500 <- getEnrichmentTables(job_tmp, ontology = availableOntologies(job_tmp))

	# Significant terms analysis - processing

		processing_up_100 <- enrichmentsSumUp(disregulatedGenes_processing_up_100, th=0.001)
		processing_up_500 <- enrichmentsSumUp(disregulatedGenes_processing_up_500, th=0.001)
		
		processing_down_100 <- enrichmentsSumUp(disregulatedGenes_processing_down_100, th=0.001)
		processing_down_500 <- enrichmentsSumUp(disregulatedGenes_processing_down_500, th=0.001)
	
		processing_both <- commonGOterms(a = collapsedGOterms(a = processing_up_100,b = processing_up_500)
										  , b = collapsedGOterms(a = processing_down_100,b = processing_down_500))
		processing_both
	
		differentialGOterms(collapsedGOterms(a = processing_up_100,b = processing_up_500),processing_both)
	
		differentialGOterms(collapsedGOterms(a = processing_down_100,b = processing_down_500),processing_both)

	# degradation UP - WT > KO

		# 100
			genes_tmp <- names(sort(degradation_FC_0,decreasing=TRUE))[1:100]
		
			genesRanges_tmp <- select(mart, keys=genes_tmp, columns=c("chromosome_name","start_position","end_position"),keytype="entrezgene")
			colnames(genesRanges_tmp) <- c("chr","start","end")
		
			genesRanges_tmp$chr <- paste0("chr",genesRanges_tmp$chr)
		
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166349",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165754",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166291",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166314",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165789",]
		
			job_tmp <- submitGreatJob(genesRanges_tmp, species = "mm9",request_interval = 0)
			disregulatedGenes_degradation_up_100 <- getEnrichmentTables(job_tmp, ontology = availableOntologies(job_tmp))
		
		# 500
			genes_tmp <- names(sort(degradation_FC_0,decreasing=TRUE))[1:500]
		
			genesRanges_tmp <- select(mart, keys=genes_tmp, columns=c("chromosome_name","start_position","end_position"),keytype="entrezgene")
			colnames(genesRanges_tmp) <- c("chr","start","end")
		
			genesRanges_tmp$chr <- paste0("chr",genesRanges_tmp$chr)
		
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166349",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165754",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166291",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166314",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165789",]
		
			job_tmp <- submitGreatJob(genesRanges_tmp, species = "mm9",request_interval = 0)
			disregulatedGenes_degradation_up_500 <- getEnrichmentTables(job_tmp, ontology = availableOntologies(job_tmp))
	
	# degradation DOWN - WT < KO

		# 100
			genes_tmp <- names(sort(degradation_FC_0,decreasing=FALSE))[1:100]
		
			genesRanges_tmp <- select(mart, keys=genes_tmp, columns=c("chromosome_name","start_position","end_position"),keytype="entrezgene")
			colnames(genesRanges_tmp) <- c("chr","start","end")
		
			genesRanges_tmp$chr <- paste0("chr",genesRanges_tmp$chr)
		
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166349",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165754",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166291",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166314",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165789",]
		
			job_tmp <- submitGreatJob(genesRanges_tmp, species = "mm9",request_interval = 0)
			disregulatedGenes_degradation_down_100 <- getEnrichmentTables(job_tmp, ontology = availableOntologies(job_tmp))
		
		# 500
		
			genes_tmp <- names(sort(degradation_FC_0,decreasing=FALSE))[1:500]
		
			genesRanges_tmp <- select(mart, keys=genes_tmp, columns=c("chromosome_name","start_position","end_position"),keytype="entrezgene")
			colnames(genesRanges_tmp) <- c("chr","start","end")
		
			genesRanges_tmp$chr <- paste0("chr",genesRanges_tmp$chr)
		
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166349",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165754",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166291",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166314",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165789",]
		
			job_tmp <- submitGreatJob(genesRanges_tmp, species = "mm9",request_interval = 0)
			disregulatedGenes_degradation_down_500 <- getEnrichmentTables(job_tmp, ontology = availableOntologies(job_tmp))

	# Significant terms analysis - degradation

		degradation_up_100 <- enrichmentsSumUp(disregulatedGenes_degradation_up_100, th=0.001)
		degradation_up_500 <- enrichmentsSumUp(disregulatedGenes_degradation_up_500, th=0.001)
		
		degradation_down_100 <- enrichmentsSumUp(disregulatedGenes_degradation_down_100, th=0.001)
		degradation_down_500 <- enrichmentsSumUp(disregulatedGenes_degradation_down_500, th=0.001)
	
		degradation_both <- commonGOterms(a = collapsedGOterms(a = degradation_up_100,b = degradation_up_500)
									    , b = collapsedGOterms(a = degradation_down_100,b = degradation_down_500))
		degradation_both
	
		collapsedGOterms(a = degradation_up_100,b = degradation_up_500)
	
		collapsedGOterms(a = degradation_down_100,b = degradation_down_500)

### Time course analysis

# Data
	totalKO_15 <- totalKO[,"total_0.25"]
	prematureKO_15 <- prematureKO[,"preMRNA_0.25"]
	synthesisKO_15 <- synthesisKO[,"synthesis_0.25"]
	processingKO_15 <- processingKO[,"processing_0.25"]
	degradationKO_15 <- degradationKO[,"degradation_0.25"]
	
	totalWT_15 <- totalWT[,"total_0.25"]
	prematureWT_15 <- prematureWT[,"preMRNA_0.25"]
	synthesisWT_15 <- synthesisWT[,"synthesis_0.25"]
	processingWT_15 <- processingWT[,"processing_0.25"]
	degradationWT_15 <- degradationWT[,"degradation_0.25"]
	
	totalKO_60 <- totalKO[,"total_1"]
	prematureKO_60 <- prematureKO[,"preMRNA_1"]
	synthesisKO_60 <- synthesisKO[,"synthesis_1"]
	processingKO_60 <- processingKO[,"processing_1"]
	degradationKO_60 <- degradationKO[,"degradation_1"]
	
	totalWT_60 <- totalWT[,"total_1"]
	prematureWT_60 <- prematureWT[,"preMRNA_1"]
	synthesisWT_60 <- synthesisWT[,"synthesis_1"]
	processingWT_60 <- processingWT[,"processing_1"]
	degradationWT_60 <- degradationWT[,"degradation_1"]

# Kolmogorov Smirnov and Mann Whitney tests
	# Synthesis
		# KO
			suppressWarnings(format(ks.test(synthesisKO_0,synthesisKO_15)[[2]],scientific=TRUE,digits=2)) # 4.1e-04
			suppressWarnings(format(wilcox.test(synthesisKO_0,synthesisKO_15)[[3]],scientific=TRUE,digits=2)) # 1.3e-06

			suppressWarnings(format(ks.test(synthesisKO_15,synthesisKO_60)[[2]],scientific=TRUE,digits=2)) # 0e+00
			suppressWarnings(format(wilcox.test(synthesisKO_15,synthesisKO_60)[[3]],scientific=TRUE,digits=2)) # 1.4e-46
		# WT
			suppressWarnings(format(ks.test(synthesisWT_0,synthesisWT_15)[[2]],scientific=TRUE,digits=2)) # 0e+00
			suppressWarnings(format(wilcox.test(synthesisWT_0,synthesisWT_15)[[3]],scientific=TRUE,digits=2)) # 1.3e-32

			suppressWarnings(format(ks.test(synthesisWT_15,synthesisWT_60)[[2]],scientific=TRUE,digits=2)) # 2e-01
			suppressWarnings(format(wilcox.test(synthesisWT_15,synthesisWT_60)[[3]],scientific=TRUE,digits=2)) # 7.1e-01

	# Processing
		# KO
			suppressWarnings(format(ks.test(processingKO_0,processingKO_15)[[2]],scientific=TRUE,digits=2)) # 6e-02
			suppressWarnings(format(wilcox.test(processingKO_0,processingKO_15)[[3]],scientific=TRUE,digits=2)) # 7.2e-01

			suppressWarnings(format(ks.test(processingKO_15,processingKO_60)[[2]],scientific=TRUE,digits=2)) # 1.1e-03
			suppressWarnings(format(wilcox.test(processingKO_15,processingKO_60)[[3]],scientific=TRUE,digits=2)) # 2.2e-01
		# WT
			suppressWarnings(format(ks.test(processingWT_0,processingWT_15)[[2]],scientific=TRUE,digits=2)) # 0e+00
			suppressWarnings(format(wilcox.test(processingWT_0,processingWT_15)[[3]],scientific=TRUE,digits=2)) # 9.1e-25

			suppressWarnings(format(ks.test(processingWT_15,processingWT_60)[[2]],scientific=TRUE,digits=2)) # 3.7e-07
			suppressWarnings(format(wilcox.test(processingWT_15,processingWT_60)[[3]],scientific=TRUE,digits=2)) # 4.9e-01

	# Degradation
		# KO
			suppressWarnings(format(ks.test(degradationKO_0,degradationKO_15)[[2]],scientific=TRUE,digits=2)) # 0e+00
			suppressWarnings(format(wilcox.test(degradationKO_0,degradationKO_15)[[3]],scientific=TRUE,digits=2)) # 1.6e-112

			suppressWarnings(format(ks.test(degradationKO_15,degradationKO_60)[[2]],scientific=TRUE,digits=2)) # 0e+00
			suppressWarnings(format(wilcox.test(degradationKO_15,degradationKO_60)[[3]],scientific=TRUE,digits=2)) # 9e-20
		# WT
			suppressWarnings(format(ks.test(degradationWT_0,degradationWT_15)[[2]],scientific=TRUE,digits=2)) # 0e+00
			suppressWarnings(format(wilcox.test(degradationWT_0,degradationWT_15)[[3]],scientific=TRUE,digits=2)) # 4.8e-165

			suppressWarnings(format(ks.test(degradationWT_15,degradationWT_60)[[2]],scientific=TRUE,digits=2)) # 0e+00
			suppressWarnings(format(wilcox.test(degradationWT_15,degradationWT_60)[[3]],scientific=TRUE,digits=2)) # 4.8e-22

# Boxplots - Figure 2
	par(mfrow=c(2,3))
	
	boxplot(cbind("0"=synthesisWT_0,"15"=synthesisWT_15,"60"=synthesisWT_60),outline=FALSE,main="Synthesis WT",las=2,ylim=c(0,35),xlab="time[m]",notch=TRUE)
	boxplot(cbind("0"=processingWT_0,"15"=processingWT_15,"60"=processingWT_60),outline=FALSE,main="Processing WT",las=2,ylim=c(0,20),xlab="time[m]",notch=TRUE)
	boxplot(cbind("0"=degradationWT_0,"15"=degradationWT_15,"60"=degradationWT_60),outline=FALSE,main="Degradation WT",las=2,ylim=c(0,3.5),xlab="time[m]",notch=TRUE)
	
	boxplot(cbind("0"=synthesisKO_0,"15"=synthesisKO_15,"60"=synthesisKO_60),outline=FALSE,main="Synthesis KO",las=2,ylim=c(0,35),xlab="time[m]",notch=TRUE)
	boxplot(cbind("0"=processingKO_0,"15"=processingKO_15,"60"=processingKO_60),outline=FALSE,main="Processing KO",las=2,ylim=c(0,20),xlab="time[m]",notch=TRUE)
	boxplot(cbind("0"=degradationKO_0,"15"=degradationKO_15,"60"=degradationKO_60),outline=FALSE,main="Degradation KO",las=2,ylim=c(0,3.5),xlab="time[m]",notch=TRUE)
	
### Clustering and heatmap

	# Data
		# I remove 7 genes which have not defined degradation rates,
		# 1 outlier gene with extremely high degradation rate and
		# 1 outlier gene extremely expressed in WT

		bool_tmp <- apply(totalWT,1,function(r)all(r<1000)) & apply(degradationKO,1,function(r)all(is.finite(r))&all(r<1000))

		totalKO_0 <- totalKO_0[names(bool_tmp)[bool_tmp]]
		prematureKO_0 <- prematureKO_0[names(bool_tmp)[bool_tmp]]
		synthesisKO_0 <- synthesisKO_0[names(bool_tmp)[bool_tmp]]
		processingKO_0 <- processingKO_0[names(bool_tmp)[bool_tmp]]
		degradationKO_0 <- degradationKO_0[names(bool_tmp)[bool_tmp]]
		
		totalWT_0 <- totalWT_0[names(bool_tmp)[bool_tmp]]
		prematureWT_0 <- prematureWT_0[names(bool_tmp)[bool_tmp]]
		synthesisWT_0 <- synthesisWT_0[names(bool_tmp)[bool_tmp]]
		processingWT_0 <- processingWT_0[names(bool_tmp)[bool_tmp]]
		degradationWT_0 <- degradationWT_0[names(bool_tmp)[bool_tmp]]

		totalKO_15 <- totalKO_15[names(bool_tmp)[bool_tmp]]
		prematureKO_15 <- prematureKO_15[names(bool_tmp)[bool_tmp]]
		synthesisKO_15 <- synthesisKO_15[names(bool_tmp)[bool_tmp]]
		processingKO_15 <- processingKO_15[names(bool_tmp)[bool_tmp]]
		degradationKO_15 <- degradationKO_15[names(bool_tmp)[bool_tmp]]
		
		totalWT_15 <- totalWT_15[names(bool_tmp)[bool_tmp]]
		prematureWT_15 <- prematureWT_15[names(bool_tmp)[bool_tmp]]
		synthesisWT_15 <- synthesisWT_15[names(bool_tmp)[bool_tmp]]
		processingWT_15 <- processingWT_15[names(bool_tmp)[bool_tmp]]
		degradationWT_15 <- degradationWT_15[names(bool_tmp)[bool_tmp]]
		
		totalKO_60 <- totalKO_60[names(bool_tmp)[bool_tmp]]
		prematureKO_60 <- prematureKO_60[names(bool_tmp)[bool_tmp]]
		synthesisKO_60 <- synthesisKO_60[names(bool_tmp)[bool_tmp]]
		processingKO_60 <- processingKO_60[names(bool_tmp)[bool_tmp]]
		degradationKO_60 <- degradationKO_60[names(bool_tmp)[bool_tmp]]
		
		totalWT_60 <- totalWT_60[names(bool_tmp)[bool_tmp]]
		prematureWT_60 <- prematureWT_60[names(bool_tmp)[bool_tmp]]
		synthesisWT_60 <- synthesisWT_60[names(bool_tmp)[bool_tmp]]
		processingWT_60 <- processingWT_60[names(bool_tmp)[bool_tmp]]
		degradationWT_60 <- degradationWT_60[names(bool_tmp)[bool_tmp]]

	# FCs
		totalKO_15_FC <- log2(totalKO_15) - log2(totalKO_0)
		prematureKO_15_FC <- log2(prematureKO_15) - log2(prematureKO_0)
		synthesisKO_15_FC <- log2(synthesisKO_15) - log2(synthesisKO_0)
		processingKO_15_FC <- log2(processingKO_15) - log2(processingKO_0)
		degradationKO_15_FC <- log2(degradationKO_15) - log2(degradationKO_0)
		
		totalWT_15_FC <- log2(totalWT_15) - log2(totalWT_0)
		prematureWT_15_FC <- log2(prematureWT_15) - log2(prematureWT_0)
		synthesisWT_15_FC <- log2(synthesisWT_15) - log2(synthesisWT_0)
		processingWT_15_FC <- log2(processingWT_15) - log2(processingWT_0)
		degradationWT_15_FC <- log2(degradationWT_15) - log2(degradationWT_0)
		
		totalKO_60_FC <- log2(totalKO_60) - log2(totalKO_0)
		prematureKO_60_FC <- log2(prematureKO_60) - log2(prematureKO_0)
		synthesisKO_60_FC <- log2(synthesisKO_60) - log2(synthesisKO_0)
		processingKO_60_FC <- log2(processingKO_60) - log2(processingKO_0)
		degradationKO_60_FC <- log2(degradationKO_60) - log2(degradationKO_0)
		
		totalWT_60_FC <- log2(totalWT_60) - log2(totalWT_0)
		prematureWT_60_FC <- log2(prematureWT_60) - log2(prematureWT_0)
		synthesisWT_60_FC <- log2(synthesisWT_60) - log2(synthesisWT_0)
		processingWT_60_FC <- log2(processingWT_60) - log2(processingWT_0)
		degradationWT_60_FC <- log2(degradationWT_60) - log2(degradationWT_0)

	# Z - scores
		# Joint distributions
			total_m <- cbind(totalWT_0,totalKO_0)
			premature_m <- cbind(prematureWT_0,prematureKO_0)
			synthesis_m <- cbind(synthesisWT_0,synthesisKO_0)
			processing_m <- cbind(processingWT_0,processingKO_0)
			degradation_m <- cbind(degradationWT_0,degradationKO_0)
		# Median
			total_median <- median(c(total_m),na.rm=TRUE)
			premature_median <- median(c(premature_m),na.rm=TRUE)
			synthesis_median <- median(c(synthesis_m),na.rm=TRUE)
			processing_median <- median(c(processing_m),na.rm=TRUE)
			degradation_median <- median(c(degradation_m),na.rm=TRUE)
		# Dispersion
			total_IQR <- IQR(c(total_m),na.rm=TRUE)
			premature_IQR <- IQR(c(premature_m),na.rm=TRUE)
			synthesis_IQR <- IQR(c(synthesis_m),na.rm=TRUE)
			processing_IQR <- IQR(c(processing_m),na.rm=TRUE)
			degradation_IQR <- IQR(c(degradation_m),na.rm=TRUE)
		
		total_z <- (total_m - total_median)/total_IQR
		premature_z <- (premature_m - premature_median)/premature_IQR
		synthesis_z <- (synthesis_m - synthesis_median)/synthesis_IQR
		processing_z <- (processing_m - processing_median)/processing_IQR
		degradation_z <- (degradation_m - degradation_median)/degradation_IQR

	data <- cbind(total_z
				, premature_z
				, synthesis_z
				, processing_z
				, degradation_z
				, "totalWT_15" = totalWT_15_FC
				, "totalKO_15" = totalKO_15_FC
				, "totalWT_60" = totalWT_60_FC
				, "totalKO_60" = totalKO_60_FC
				, "prematureWT_15" = prematureWT_15_FC
				, "prematureKO_15" = prematureKO_15_FC
				, "prematureWT_60" = prematureWT_60_FC
				, "prematureKO_60" = prematureKO_60_FC
				, "synthesisWT_15" = synthesisWT_15_FC
				, "synthesisKO_15" = synthesisKO_15_FC
				, "synthesisWT_60" = synthesisWT_60_FC
				, "synthesisKO_60" = synthesisKO_60_FC
				, "processingWT_15" = processingWT_15_FC
				, "processingKO_15" = processingKO_15_FC
				, "processingWT_60" = processingWT_60_FC
				, "processingKO_60" = processingKO_60_FC
				, "degradationWT_15" = degradationWT_15_FC
				, "degradationKO_15" = degradationKO_15_FC
				, "degradationWT_60" = degradationWT_60_FC
				, "degradationKO_60" = degradationKO_60_FC)
	# Boxplots - Figure S2
		par(mar=c(10.1,4.1,2.1,2.1))
		boxplot(data,outline=FALSE,las=2,notch=TRUE)

	# Clustering metrics
		kmeansOut <- lapply(2:15, function(centers){print(paste('Trying',centers,'clusters...')); kmeans(data, centers=centers,iter.max=35)})

		enrichments <- lapply(kmeansOut[1:14],function(i)
		{
			ids <- lapply(unique(i$cluster),function(c){names(i$cluster)[i$cluster==c]})
			topGOres2(ids = ids, ontology = "BP", Pthr = 1e-05, maxN = 5000, minN = 5, orgdb=org.Mm.eg.db, allEG = keys(org.Mm.eg.db), showTerms = NULL)
		})

		# Figure S3
		par(mfrow=c(1,3))

		# Small clusters
		plot(2:15,sapply(kmeansOut,function(i)table(table(i[[1]])<15)["TRUE"]),type="l",xlab="Percentage of clusters",ylab="Number of clusters < 25"); abline(v=11,lty=3)
		# Clusters similarity
		plot(2:15,sapply(enrichments,function(c){
				goTerms <- sapply(c[!sapply(c,is.null)],function(i)i[[1]])
				median(sapply(seq_along(goTerms),function(j){length(setdiff(unlist(goTerms[j]),unlist(goTerms[-j])))/length(unlist(goTerms[j]))}))
			}),type="l",xlab="Number of clusters",ylab="Mean percentage of unique terms"); abline(v=11,lty=3)
		# Average significance
		plot(2:15,sapply(enrichments,function(c){
				mean(sapply(c[!sapply(c,is.null)],function(i)
				{
					mean(log10(i$classic))
				}))
			}),type="l",xlab="Number of clusters",ylab="Median terms significance"); abline(v=11,lty=3)

	# Final clustering with 11 groups
		set.seed(1)
		clustering <- kmeans(data, centers=11, iter.max=35)
		ids <- lapply(unique(clustering$cluster),function(c){names(clustering$cluster)[clustering$cluster==c]})

		orders <- lapply(ids,function(g)g[hclust(dist(data[g,]))$ord])
		orders[[11]] <- ids[[11]]
	
	# Graphication - Figure S4

		x_sat <- data
		x_sat[x_sat<(-1.5)] <- (-1.5)
		x_sat[x_sat>(1.5)] <- (1.5)
	
		x_sat <- x_sat[unlist(orders),]
	
		x_sat <- colsInsert(x_sat,seq(1,30,by=2)[-1])
		x_sat <- rowsInsert(x_sat,c(sapply(seq_along(orders)[-1],function(i){sum(sapply(orders,length)[1:i-1])})+1))

		pheatmap(x_sat
				,cluster_rows=FALSE
				,cluster_cols=FALSE
				,show_rownames=FALSE
				,color = colorRampPalette(c("blue","white","red"))(20)
				,breaks = seq(-1.5,1.5,length.out=21)
				,width=10
				,height=10)

	# Functional analysis
		clustersEnrichments <- lapply(ids,function(genes_tmp)
		{
			genesRanges_tmp <- select(mart, keys=genes_tmp, columns=c("chromosome_name","start_position","end_position"),keytype="entrezgene")
			colnames(genesRanges_tmp) <- c("chr","start","end")
			
			genesRanges_tmp$chr <- paste0("chr",genesRanges_tmp$chr)
			
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166349",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165754",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166291",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_166314",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165789",]
			genesRanges_tmp <- genesRanges_tmp[genesRanges_tmp[,1]!="chrNT_165789",]
	
			job_tmp <- submitGreatJob(genesRanges_tmp, species = "mm9",request_interval = 0)
			disregulatedGenes_tmp <- getEnrichmentTables(job_tmp, ontology = availableOntologies(job_tmp))
		
		})

	# Sumup of the results
		sumUps <- sapply(clustersEnrichments,function(i)enrichmentsSumUp(i,10^-7))


